function entry = option_entry(cmd,name,description,default,validator,parser)
%   Makes an entry that allows the user to set options
%   These options persist in the menu and can be accessed by any entry
%
%   entry = option_entry(cmd,name,description,default)
%   entry = option_entry(cmd,name,description,default,validator)
%
%   Input Arguments:
%       cmd - The sequence used to change the option
%       name - The name of the option
%       description - A description of the option.  This will also serve
%                     As the prompt to the user
%       default - default value for the option, as a value
%       validator - ret = validator(x) x is the value entered by the user
%                   return false if the value is unacceptable
%       parser - ret = parser(x) x is the value entered by the user
%                   converts the user string into the option value,
%                   defaults to simply returning the string
%   
%       action
%   Output Arguments:
%       entry - a menu entry for setting the option. Additional fields:
%               entry.name - the name of the option
%               entry.default - the default value
    nin = nargin;
    if nin < 5
        validator = @(x) true;
    end
    if nin < 6
        parser = @(x) x;
    end
    
    function ret = option_action(menu,cmd) %#ok<INUSD>
        %this is the action when an option is executed.
        % ret.value is the value of the option
        % ret.str is a nice string value of the option
        rmenu = root_menu(menu);
        ret.name = name;
        disp(['Option ',name,' = ', rmenu.options.(name).str]);
        ret.str = input([description,': '],'s');
        if isempty(ret.str)
            ret.value = rmenu.options.(name).value;
            ret.str = rmenu.options.(name).str;
        elseif ~validator(ret.str)
            disp('Invalid Input. Option not changed.');
            ret.value = rmenu.options.(name).value;
            ret.str = rmenu.options.(name).str;
        else
            ret.value = parser(ret.str);
        end
    end
    entry = menu_entry(cmd,[name,': ',description],@option_action);
    entry.default.value = parser(default);
    entry.default.str = default;    
    entry.name = name;
end